
int cS=18;

int nbX=18;
int nbY=18;

int squaresToMerge=2;

float transition = 1;

int currentPlayer=0;

Cell[][] cells = new Cell[nbX][nbY];

void setup() {
  size(cS*nbX, cS*nbY);
  for (int x=0;x<nbX;x++) {
    for (int y=0;y<nbY;y++) {    
      cells[x][y]=new Cell(x, y, 0);
    }
  }
  noStroke();
}

void draw() {
  background(0);
  // draw outer squares
  stroke(0x80);
  if (transition<=1) {
    for (int x=0;x<nbX;x+=squaresToMerge) {
      for (int y=0;y<nbY;y+=squaresToMerge) {
        int[] toMerge = new int[squaresToMerge*squaresToMerge];
        int arrayI=0;
        for (int x2=0;x2<squaresToMerge;x2+=1) {
          for (int y2=0;y2<squaresToMerge;y2+=1) {
            toMerge[arrayI]=cells[x+x2][y+y2].state;
            arrayI++;
          }
        }
        int colorToFillWith = mergeColors(toMerge);
        if (colorToFillWith==0) noFill();
        if (colorToFillWith==1) fill(0xFF, 0, 0, 0xA0);
        if (colorToFillWith==2) fill(0, 0xFF, 0, 0xA0);
        rect(x*cS, y*cS, cS*squaresToMerge, cS*squaresToMerge);
      }
    }
  }
  noStroke();
  // draw actual squares
  for (int x=0;x<nbX;x++) {
    for (int y=0;y<nbY;y++) {    
      cells[x][y].display(transition);
    }
  }
  transition=max(transition-0.05, 1);
}

class Cell {
  int x;
  int y;
  int state;
  int margin=2;

  Cell(int x, int y, int state) {
    this.x=x;
    this.y=y;
    this.state=state;
  }

  void display(float transition) {
    if (state==0) fill(0x50, 0x50, 0x50);
    if (state==1) fill(0xFF, 0, 0);
    if (state==2) fill(0, 0xFF, 0);
    if (transition>1) {
      pushMatrix();
      translate((float)nbX*cS/2, (float)nbY*cS/2);
      scale(transition, transition);
      translate(-(float)nbX*cS/2+((float)cS/2*transition/squaresToMerge), -(float)nbY*cS/2+((float)cS/2*transition/squaresToMerge));
      rect(x*cS, y*cS, cS, cS);
      popMatrix();
    }
    else rect(x*cS+margin, y*cS+margin, cS-margin*2, cS-margin*2);
  }

  void setState(int s) {
    state = s;
  }
}

void mousePressed() {
  int newX=floor((float)mouseX/cS);
  int newY=floor((float)mouseY/cS);
  cells[newX][newY].setState(currentPlayer+1);
  currentPlayer=(currentPlayer+1)%2;
  checkSandwitch(newX, newY);
}

void keyPressed() {
  if (keyCode==ENTER) {
    Cell[][] newCells = new Cell[nbX][nbY];
    for (int x=0;x<nbX;x++) {
      for (int y=0;y<nbY;y++) {    
        newCells[x][y]= new Cell(x, y, 0);
      }
    }
    for (int x=0;x<nbX;x+=squaresToMerge) {
      for (int y=0;y<nbY;y+=squaresToMerge) {
        int[] toMerge = new int[squaresToMerge*squaresToMerge];
        int arrayI=0;
        for (int x2=0;x2<squaresToMerge;x2+=1) {
          for (int y2=0;y2<squaresToMerge;y2+=1) {
            toMerge[arrayI]=cells[x+x2][y+y2].state;
            arrayI++;
          }
        }
        int newX = floor((((float)nbX/2)+((float)x-((float)nbX/2))/squaresToMerge));
        int newY = floor((((float)nbY/2)+((float)y-((float)nbY/2))/squaresToMerge));
        newCells[newX][newY]= new Cell(newX, newY, mergeColors(toMerge));
      }
    }
    cells=newCells;
    transition=squaresToMerge;
  }
}

int mergeColors(int[] squares) {
  int[] colors = new int[3];
  for (int i=0;i<squares.length;i++) {
    colors[squares[i]]+=1;
  }
  if (colors[1]>colors[2]) return 1;
  if (colors[2]>colors[1]) return 2;  
  return 0;
}

void checkSandwitch(int x, int y) {
  int x2=1;
  int upTo=0;
  boolean ok=false;
  while (x+x2<nbX && !ok) {
    if (cells[x+x2][y].state==0) ok=true;
    if (cells[x+x2][y].state==cells[x][y].state) {
      upTo=x2;
      ok=true;
    }
    x2++;
  }
  for (int x3=0;x3<upTo;x3++) {
    cells[x+x3][y].state=cells[x][y].state;
  }
  //
  x2=-1;
  upTo=0;
  ok=false;
  while (x+x2>=0 && !ok) {
    if (cells[x+x2][y].state==0) ok=true;
    if (cells[x+x2][y].state==cells[x][y].state) {
      upTo=x2;
      ok=true;
    }
    x2--;
  }
  for (int x3=0;x3>upTo;x3--) {
    cells[x+x3][y].state=cells[x][y].state;
  }
  //
  int y2=1;
  upTo=0;
  ok=false;
  while (y+y2<nbY && !ok) {
    if (cells[x][y+y2].state==0) ok=true;
    if (cells[x][y+y2].state==cells[x][y].state) {
      upTo=y2;
      ok=true;
    }
    y2++;
  }
  for (int y3=0;y3<upTo;y3++) {
    cells[x][y+y3].state=cells[x][y].state;
  }
  //
  y2=-1;
  upTo=0;
  ok=false;
  while (y+y2>=0 && !ok) {
    if (cells[x][y+y2].state==0) ok=true;
    if (cells[x][y+y2].state==cells[x][y].state) {
      upTo=y2;
      ok=true;
    }
    y2--;
  }
  for (int y3=0;y3>upTo;y3--) {
    cells[x][y+y3].state=cells[x][y].state;
  }
}

void checkAllSandwitches() {
  Cell[][] newCells = new Cell[nbX][nbY];
  for (int x=0;x<nbX;x++) {
    for (int y=0;y<nbY;y++) {
      newCells[x][y]= new Cell(x, y, cells[x][y].state);
    }
  }
  for (int x=0;x<nbX-2;x++) {
    for (int y=0;y<nbY-2;y++) {
      if (cells[x][y].state==1 && cells[x+1][y].state==2 && cells[x+2][y].state==1) newCells[x+1][y].state=1;
      if (cells[x][y].state==1 && cells[x][y+1].state==2 && cells[x][y+2].state==1) newCells[x][y+1].state=1;
      if (cells[x][y].state==2 && cells[x+1][y].state==1 && cells[x+2][y].state==2) newCells[x+1][y].state=2;
      if (cells[x][y].state==2 && cells[x][y+1].state==1 && cells[x][y+2].state==2) newCells[x][y+1].state=2;
    }
  }
  cells=newCells;
}

